# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class EntitiesController < Hermes::Api::Dmo::ApplicationController
        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          als_id
          title
          siret
          siren
          is_root
          root_id
          parent_id
          id_rex
          id_cristal
          automated_request_validations
          delegated_validations
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          total_spent
          points_balance
          applicant_ids
          housing_offer_ids
          lessor_id
          group_id
          supervisor_group_id
          manager_group_id
          address
          postal_code
          district
          country
          latitude
          longitude
          auto_validation
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          title
          siret
          siren
          parent_id
          id_rex
          id_cristal
          automated_request_validations
          delegated_validations
          priorities_enabled
          in_peec
          achieved_goal
          spent_on_completed_requests
          spent_on_ongoing_requests
          spent_on_other
          annual_stock
          applicant_ids
          housing_offer_ids
          lessor_id
          address
          postal_code
          district
          country
          latitude
          longitude
          auto_validation
        ].freeze
        mutate_params_only(*MUTATE_PARAMS)

        def index
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          @query_precision ||= {}
          if params[:housing_offer_ids]
            housing_entity_ids = Hermes::HousingEntityOffer.only(:id, :housing_offer_id)
                                                           .where(
                                                             housing_offer_id: params[:housing_offer_ids]
                                                           ).pluck(:id)
            @query_precision[:housing_entity_offer_ids] = housing_entity_ids
          end
          super
        end

        def can_create_tier_account
          Hermes::Entity.readable_by(current_user).find_by(id: @instance.id) && 
          (
            current_user.auth_roles&.include?('entreprise') || 
            current_user.auth_roles&.include?('entreprise administrateur')
          )
        end

        def checks_before_create_account
          @instance = Hermes::Entity.find_by(id: params[:id]) if params[:id]
          return [:not_found, :instance_not_found] unless @instance
          return [:unauthorized, :no_right_on_this_entity] if !can_create_tier_account
          if [:id, :first_name, :last_name, :email, :gender, :phone_number].any? { |p| !params.has_key?(p)  }
            return [:unprocessable_entity, :user_information_missing] 
          end
        end 

        def check_for_beiam_error(user)
          user.instance_variable_get(:@create_user_response).code != 201 &&
          user.instance_variable_get(:@create_user_response).code != 409
        end 

        def beiam_error(user)
          {
            json: user.instance_variable_get(:@create_user_response).body, 
            status: user.instance_variable_get(:@create_user_response).code
          }
        end

        def create_user_from_tier_account(role, group_ids = [])
          
          roles_mapping = {
            "demandeur" => "salarie",
            "collaborateur entreprise" => "collaborateur entreprises",
            "entreprise" => "entreprise",
            "entreprise administrateur" => "entreprise administrateur",
          }
          
          group_id = "g_" + @instance.group_id
          manager_group_id = "g_" + @instance.manager_group_id
          supervisor_group_id = "g_" + @instance.supervisor_group_id
          
          rw = {
            'demandeur' => [manager_group_id, supervisor_group_id, group_id],
            'collaborateur entreprise' => [manager_group_id, supervisor_group_id],
            'entreprise' =>  [manager_group_id],
            'entreprise administrateur' => [manager_group_id],
          }

          User.create(
            first_name: params[:first_name],
            last_name: params[:last_name],
            email: params[:email],
            phone_number: params[:phone_number],
            login: params[:login],
            writers: rw[role],
            readers: rw[role],
            auth_roles: [role],
            group_ids: group_ids,
            password: Devise.friendly_token[0,20],
            provider: HERMES_CONFIG['identity_provider_key'],
            create_user_data: {
              gender: params[:gender],
              lastName: params[:last_name],
              firstName: params[:first_name],
              email: params[:email],
              login: params[:login],
              mobileNumber: params[:phone_number],
              roles: [roles_mapping[role]]
            },
          )
        end

        def create_applicant
          return render_json_error(*checks_before_create_account) if checks_before_create_account

          user = create_user_from_tier_account('demandeur')

          return render json: user.errors.messages, status: 400 if user.invalid?
          return render(beiam_error(user)) if check_for_beiam_error(user)

          rw = [@instance.group_id, @instance.manager_group_id, @instance.supervisor_group_id]

          controller = Hermes::Api::Dmo::ApplicantsController.new
          controller.params = {
            data: {
              attributes: params.except(:id).merge(
                main_user_id: user.id, 
                writers: rw,
                readers: rw,
              ) 
            }
          }
          controller.instance_variable_set(:@user, @user)
          controller.create_without_render
          return respond_with_errors(controller.instance_variable_get(:@instance)) unless controller.instance_variable_get(:@instance)
          
          controller.instance_variable_get(:@instance).entity_id = @instance.id
          controller.instance_variable_get(:@instance).save!
          render(jsonapi: controller.instance_variable_get(:@instance), serializer: controller.instance_variable_get(:@serializer), status: :created)
        end

        def create_user
          return render_json_error(*checks_before_create_account) if checks_before_create_account

          user = create_user_from_tier_account('collaborateur entreprise', [@instance.group_id])

          return render json: user.errors.messages, status: 400 if user.invalid?
          return render(beiam_error(user)) if check_for_beiam_error(user)

          render(jsonapi: user, serializer: UserSerializer, status: :created)
        end
        
        def create_supervisor
          return render_json_error(*checks_before_create_account) if checks_before_create_account
          
          user = create_user_from_tier_account('entreprise', [@instance.supervisor_group_id])

          return render json: user.errors.messages, status: 400 if user.invalid?
          return render(beiam_error(user)) if check_for_beiam_error(user)

          render(jsonapi: user, serializer: UserSerializer, status: :created)
        end

      end
    end
  end
end
